@@ -137,13 +137,16 @@ class PureAttention(base.Layer):
       x = x.reshape((batch_size, -1, n_heads * d_head))
       return x
 
-    per_head_results = DotProductAttention(_split_into_heads(q),
-                                           _split_into_heads(k),
-                                           _split_into_heads(v),
-                                           mask,
-                                           dropout=self._dropout,
-                                           mode=self._mode,
-                                           rng=self.rng)
+    per_head_results, dots = DotProductAttention(
+        _split_into_heads(q),
+        _split_into_heads(k),
+        _split_into_heads(v),
+        mask,
+        dropout=self._dropout,
+        mode=self._mode,
+        rng=self.rng)
+    if self._mode == 'viz':
+      self.state = dots
     merged_results = _merge_heads(per_head_results)
     return (merged_results, mask)
 
@@ -190,7 +193,7 @@ def DotProductAttention(queries, keys, values, mask, dropout, mode, rng):
     keep = fastmath.random.bernoulli(rng, 1.0 - dropout, dots.shape)
     dots = jnp.where(keep, dots / (1.0 - dropout), jnp.zeros_like(dots))
   out = jnp.matmul(dots, values)
-  return out
+  return out, dots
 
 
 def CausalAttention(d_feature, n_heads=1, dropout=0.0, mode='train'):
@@ -276,8 +279,10 @@ class DotProductCausalAttention(base.Layer):
         mask = np.tril(
             np.ones((1, mask_size, mask_size), dtype=np.bool_), k=0)
 
-    res = DotProductAttention(
+    res, dots = DotProductAttention(
         q, k, v, mask, dropout=self._dropout, mode=self._mode, rng=self.rng)
+    if self._mode == 'viz':
+      self.state = dots
     return res
 
   def init_weights_and_state(self, input_signature):
@@ -409,28 +414,44 @@ def _fast_inference_init_state(input_signature, buffer_length):
   k = zeros_for(batch_size, input_signature[1])
   v = zeros_for(batch_size, input_signature[2])
   mask = jnp.zeros((batch_size, 1, buffer_length))
-  seq_indices = jnp.zeros((batch_size,), dtype=jnp.int32)
-  return (k, v, mask, seq_indices)
+  return (k, v, mask, jnp.array(0))
 
 
 def _fast_inference_update_state(inputs, state):
-  """Updates state of a causal attention layer for fast inference."""
+  """Updates state of a causal attention layer for fast inference.
+
+  The layer state stores tensors with cached values of keys and values,
+  as well as the mask and an index. To make shapes static, keys and values
+  in the state are long, and the index indicates where the new keys and values
+  from inputs need to be appended. Mask ensures that attention will only look
+  at keys upto index.
+
+  During update, we append new_keys and new_values to keys and values at
+  position given by index. We also update mask (which starts as all-0s) to
+  be 1 at the new keys positions. And we increment index by length of new keys.
+
+  Args:
+    inputs: a triple (new_queries, new_keys, new_values)
+    state: layer state with (keys, values, mask, index)
+
+  Returns:
+    Updated state.
+  """
   if fastmath.backend_name() != 'jax':
     raise ValueError(f'JAX backend is required in predict mode, but found '
                      f'backend ({fastmath.backend_nameO()}).')
-  for x in inputs:
-    if x.shape[1] != 1:
-      raise ValueError(f'In predict mode, input sequence must have length 1, '
-                       f'instead has length {x.shape[1]}.')
-  # Fast inference: run with only 1 query in each step, storing the sequence
+
+  # Fast inference: run step-by-step, storing the sequence
   # of keys and values calculated so far in state.
   (_, new_k, new_v) = inputs
-  (ks, vs, mask, seq_indices) = state
-  batch_indices = jnp.arange(ks.shape[0])
-  ks = jax.ops.index_update(
-      ks, jax.ops.index[batch_indices, seq_indices, :], new_k[:, 0, :])
-  vs = jax.ops.index_update(
-      vs, jax.ops.index[batch_indices, seq_indices, :], new_v[:, 0, :])
-  mask = jax.ops.index_update(
-      mask, jax.ops.index[batch_indices, :, seq_indices], 1)
-  return (ks, vs, mask, seq_indices + 1)
+  length = new_k.shape[1]
+  (ks, vs, mask, idx) = state
+  # TODO(lukaszkaiser): benchmark speed and decide if using a separate code path
+  # with index_update when length == 1 is worth it.
+  # Keys and values are of shape [batch_size, length, d_kv].
+  ks = jax.lax.dynamic_update_slice_in_dim(ks, new_k, idx, axis=1)
+  vs = jax.lax.dynamic_update_slice_in_dim(vs, new_v, idx, axis=1)
+  # Mask is of shape [batch_size, 1 (for heads), length].
+  new_mask = jnp.ones((mask.shape[0], mask.shape[1], length))
+  mask = jax.lax.dynamic_update_slice_in_dim(mask, new_mask, idx, axis=2)
+  return (ks, vs, mask, idx + length)
