@@ -221,16 +221,26 @@ def Transformer(input_vocab_size,
     A Transformer model as a layer that maps from a source, target pair to
     activations over a vocab set.
   """
-  def PositionalEncoder(vocab_size):  # tokens --> vectors
+  def Embedder(vocab_size):  # tokens --> vectors
     return [
         tl.Embedding(vocab_size, d_model),
         tl.Dropout(rate=dropout, shared_axes=dropout_shared_axes, mode=mode),
-        tl.PositionalEncoding(max_len=max_len),
     ]
 
-  in_encoder = PositionalEncoder(input_vocab_size)
-  out_encoder = (in_encoder if output_vocab_size is None
-                 else PositionalEncoder(output_vocab_size))
+  in_embedder = Embedder(input_vocab_size)
+  out_embedder = (in_embedder if output_vocab_size is None
+                  else Embedder(output_vocab_size))
+
+  # Positional encoding are not shared between encoder and decoder.
+  # Since encoder doesn't run stepwise, we do not use predict mode there.
+  encoder_mode = 'eval' if mode == 'predict' else mode
+  in_encoder = in_embedder + [
+      tl.PositionalEncoding(max_len=max_len, mode=encoder_mode)
+  ]
+  out_encoder = out_embedder + [
+      tl.PositionalEncoding(max_len=max_len, mode=mode)
+  ]
+
   if output_vocab_size is None:
     output_vocab_size = input_vocab_size
 
@@ -264,7 +274,7 @@ def Transformer(input_vocab_size,
 
       # Decode.
       tl.Select([2, 1, 0]),               # tok_d masks vec_e .....
-      tl.ShiftRight(),                    # tok_d ..... ..... .....
+      tl.ShiftRight(mode=mode),           # tok_d ..... ..... .....
       out_encoder,                        # vec_d ..... ..... .....
       tl.Branch(
           [], tl.EncoderDecoderMask()),   # vec_d masks ..... .....
@@ -361,7 +371,7 @@ def TransformerNoEncDecAttention(input_vocab_size,
 
       # Decode.
       tl.Select([3, 1, 0, 2]),          #  tok_d vec_e mask_e tok_e tok_d
-      tl.ShiftRight(),                  # stok_d vec_e mask_e tok_e tok_d
+      tl.ShiftRight(mode=mode),         # stok_d vec_e mask_e tok_e tok_d
       tl.Branch(
           [],
           _MaskOfRightShiftedArray()
