@@ -23,6 +23,7 @@ import gym
 import numpy as np
 import tensorflow as tf
 
+from trax import data
 from trax import fastmath
 from trax import layers as tl
 from trax import shapes
@@ -146,7 +147,7 @@ class ActorCriticTrainer(rl_training.PolicyTrainer):
       # If needed, create value_output_dir and missing parent directories.
       if not tf.io.gfile.isdir(value_output_dir):
         tf.io.gfile.makedirs(value_output_dir)
-    self._value_inputs = supervised.Inputs(
+    self._value_inputs = data.inputs.Inputs(
         train_stream=lambda _: self.value_batches_stream())
     self._value_trainer = supervised.Trainer(
         model=value_model,
@@ -377,13 +378,13 @@ def _copy_model_weights_and_state(  # pylint: disable=invalid-name
 ):
   """Copy model weights[start:end] from from_trainer to to_trainer."""
   from_weights = from_trainer.model_weights
-  to_weights = to_trainer.model_weights
+  to_weights = list(to_trainer.model_weights)
   shared_weights = from_weights[start:end]
   to_weights[start:end] = shared_weights
   to_trainer.model_weights = to_weights
 
   from_state = from_trainer.model_state
-  to_state = to_trainer.model_state
+  to_state = list(to_trainer.model_state)
   shared_state = from_state[start:end]
   to_state[start:end] = shared_state
   to_trainer.model_state = to_state
