@@ -24,7 +24,7 @@ import gym
 import numpy as np
 
 from trax import fastmath
-from trax.supervised import trainer_lib
+from trax.supervised import training
 
 
 class _TimeStep(object):
@@ -418,21 +418,21 @@ class RLTask:
     """Initialize trajectories task from file."""
     # We assume that this is a dump generated by Trax
     dictionary_file = os.path.join(initial_trajectories_path, dictionary_file)
-    dictionary = trainer_lib.unpickle_from_file(dictionary_file, gzip=False)
+    dictionary = training.unpickle_from_file(dictionary_file, gzip=False)
     # TODO(henrykm): as currently implemented this accesses only
     # at most the last n_replay_epochs - this should be more flexible
     epochs_to_load = dictionary['all_epochs'][start_epoch_to_load:]
 
     all_trajectories = []
     for epoch in epochs_to_load:
-      trajectories = trainer_lib.unpickle_from_file(
+      trajectories = training.unpickle_from_file(
           self._epoch_filename(dictionary_file, epoch), gzip=True)
       all_trajectories += trajectories
     return all_trajectories
 
   def init_from_file(self, file_name):
     """Initialize this task from file."""
-    dictionary = trainer_lib.unpickle_from_file(file_name, gzip=False)
+    dictionary = training.unpickle_from_file(file_name, gzip=False)
     self._n_trajectories = dictionary['n_trajectories']
     self._n_interactions = dictionary['n_interactions']
     self._max_steps = dictionary['max_steps']
@@ -440,7 +440,7 @@ class RLTask:
     epochs_to_load = dictionary['all_epochs'][-self._n_replay_epochs:]
 
     for epoch in epochs_to_load:
-      trajectories = trainer_lib.unpickle_from_file(
+      trajectories = training.unpickle_from_file(
           self._epoch_filename(file_name, epoch), gzip=True)
       self._trajectories[epoch] = trajectories
     self._saved_epochs_unchanged = epochs_to_load
@@ -451,9 +451,9 @@ class RLTask:
     epochs_to_save = [e for e in self._trajectories.keys()
                       if e not in self._saved_epochs_unchanged]
     for epoch in epochs_to_save:
-      trainer_lib.pickle_to_file(self._trajectories[epoch],
-                                 self._epoch_filename(file_name, epoch),
-                                 gzip=True)
+      training.pickle_to_file(self._trajectories[epoch],
+                              self._epoch_filename(file_name, epoch),
+                              gzip=True)
     # Now save the list of epochs (so the trajectories are already there,
     # even in case of preemption).
     dictionary = {'n_interactions': self._n_interactions,
@@ -461,7 +461,7 @@ class RLTask:
                   'max_steps': self._max_steps,
                   'gamma': self._gamma,
                   'all_epochs': list(self._trajectories.keys())}
-    trainer_lib.pickle_to_file(dictionary, file_name, gzip=False)
+    training.pickle_to_file(dictionary, file_name, gzip=False)
 
   def play(self, policy, max_steps=None):
     """Play an episode in env taking actions according to the given policy."""
